library nb_utils;

import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:nb_utils/src/colors.dart';

export 'src/colors.dart';
export 'src/context_extensions.dart';
export 'src/date_time_utils.dart';
export 'src/livestream.dart';
export 'src/network_utils.dart';
export 'src/number_extensions.dart';
export 'src/scroll_extensions.dart';
export 'src/shared_pref.dart';
export 'src/string_extensions.dart';
export 'src/text_extensions.dart';
export 'src/time_formatter.dart';
export 'src/widget_extensions.dart';
export 'src/widgets.dart';

/// Go back to previous screen.
finish(context) => Navigator.pop(context);

/// Go to new screen with provided screen tag.
///
/// ```dart
/// launchNewScreen(context, '/HomePage');
/// ```
launchNewScreen(context, tag) => Navigator.of(context).pushNamed(tag);

/// Removes all previous screens from the back stack and redirect to new screen with provided screen tag
///
/// ```dart
/// launchNewScreenWithNewTask(context, '/HomePage');
/// ```
launchNewScreenWithNewTask(context, tag) =>
    Navigator.of(context).pushNamedAndRemoveUntil(tag, (r) => false);

/// Returns Color from hex String.
///
/// ```dart
/// Color color = getColorFromHex('#E5E5E5');
/// ```
Color getColorFromHex(String hexColor) {
  try {
    return Color(int.parse(hexColor.substring(1, 7), radix: 16) + 0xFF000000);
  } catch (e) {
    print(e);
    return Colors.white;
  }
}

/// Hide soft keyboard
hideKeyboard(context) => FocusScope.of(context).requestFocus(FocusNode());

mixin AfterLayoutMixin<T extends StatefulWidget> on State<T> {
  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance
        .addPostFrameCallback((_) => afterFirstLayout(context));
  }

  void afterFirstLayout(BuildContext context);
}

extension BooleanExtensions on bool {
  /// Validate given bool is not null and returns given value if null.
  bool validate({bool value = false}) => this ?? value;

  Widget checkNull({Widget child = const SizedBox()}) {
    return child;
  }
}

// Styles

boldTextStyle(
    {int size = 16,
    Color color = textPrimaryColor,
    FontWeight weight = FontWeight.bold}) {
  return TextStyle(
    fontSize: size.toDouble(),
    color: color,
    fontWeight: weight,
  );
}

//
primaryTextStyle(
    {int size = 16,
    Color color = textPrimaryColor,
    FontWeight weight = FontWeight.normal}) {
  return TextStyle(fontSize: size.toDouble(), color: color, fontWeight: weight);
}

//
secondaryTextStyle(
    {int size = 14,
    Color color = textSecondaryColor,
    FontWeight weight = FontWeight.normal}) {
  return TextStyle(fontSize: size.toDouble(), color: color, fontWeight: weight);
}

//

BoxDecoration boxDecorationWithRoundedCorners({
  Color backgroundColor = whiteColor,
  BorderRadius borderRadius = const BorderRadius.all(Radius.circular(8.0)),
  LinearGradient gradient,
  BoxBorder border,
  List<BoxShadow> boxShadow,
  DecorationImage decorationImage,
  BoxShape boxShape = BoxShape.rectangle,
}) {
  return BoxDecoration(
    color: backgroundColor,
    borderRadius: borderRadius,
    gradient: gradient,
    border: border,
    boxShadow: boxShadow,
    image: decorationImage,
    shape: boxShape,
  );
}

BoxDecoration boxDecorationWithShadow({
  Color backgroundColor = whiteColor,
  Color shadowColor = Colors.black12,
  blurRadius = 10.0,
  spreadRadius = 0.0,
  Offset offset = const Offset(0.0, 0.0),
  LinearGradient gradient,
  BoxBorder border,
  List<BoxShadow> boxShadow,
  DecorationImage decorationImage,
  BoxShape boxShape = BoxShape.rectangle,
  BorderRadius borderRadius,
}) {
  return BoxDecoration(
    boxShadow: [
      BoxShadow(
        color: shadowColor,
        blurRadius: blurRadius,
        spreadRadius: spreadRadius,
        offset: offset,
      ),
    ],
    color: backgroundColor,
    gradient: gradient,
    border: border,
    image: decorationImage,
    shape: boxShape,
    borderRadius: borderRadius,
  );
}

BoxDecoration boxDecorationRoundedWithShadow(
  int radiusAll, {
  Color backgroundColor = whiteColor,
  Color shadowColor = Colors.black12,
  blurRadius = 10.0,
  spreadRadius = 0.0,
  Offset offset = const Offset(0.0, 0.0),
}) {
  return BoxDecoration(
    boxShadow: defaultBoxShadow(
      shadowColor: shadowColor,
      blurRadius: blurRadius,
      spreadRadius: spreadRadius,
      offset: offset,
    ),
    color: backgroundColor,
    borderRadius: BorderRadius.all(
      Radius.circular(radiusAll.toDouble()),
    ),
  );
}

List<BoxShadow> defaultBoxShadow({
  Color shadowColor = Colors.black12,
  blurRadius = 10.0,
  spreadRadius = 0.0,
  Offset offset = const Offset(0.0, 0.0),
}) {
  return [
    BoxShadow(
      color: shadowColor,
      blurRadius: blurRadius,
      spreadRadius: spreadRadius,
      offset: offset,
    )
  ];
}

createRichText(List<TextSpan> list,
    {TextOverflow overflow = TextOverflow.ellipsis,
    int maxLines,
    TextAlign textAlign = TextAlign.left}) {
  return RichText(
      text: TextSpan(children: list),
      overflow: overflow,
      maxLines: maxLines,
      textAlign: textAlign);
}

toast(String value,
    {ToastGravity gravity = ToastGravity.BOTTOM,
    length = Toast.LENGTH_SHORT,
    Color bgColor,
    Color textColor}) {
  Fluttertoast.showToast(
      msg: value,
      gravity: gravity,
      toastLength: length,
      backgroundColor: bgColor,
      textColor: textColor);
}

toastLong(String value,
    {ToastGravity gravity = ToastGravity.BOTTOM,
    length = Toast.LENGTH_LONG,
    Color bgColor,
    Color textColor}) {
  toast(value,
      gravity: gravity, bgColor: bgColor, textColor: textColor, length: length);
}

log(value) {
  var dateTime = DateTime.now();
  print('${dateTime.hour}:${dateTime.minute}:${dateTime.second} $value');
}
